SET TERM ^ ;

CREATE OR ALTER PROCEDURE LOAD_TERMINAL_CONF(HOST_ID INTEGER,
UI_CONF_VERSION INTEGER,
CONF_TYPE INTEGER)
 RETURNS(VOICE_VERSION INTEGER,
SYSTEM_CONF_VERSION INTEGER,
TERMINAL_CONF_CFG BLOB SUB_TYPE 1,
RESULT VARCHAR(64))
 AS
declare variable SYSLOG_ADDRESS varchar(128);
declare variable SNMP_AGENT varchar(128);
declare variable SNMP_RECEIVER varchar(128);
declare variable CHANNEL_ID integer;
declare variable LOGIC varchar(32);
declare variable RE_TYPE_ID integer;
declare variable LOCAL_DESC varchar(64);
declare variable PORT integer;
declare variable URISCHEME varchar(10);
declare variable MEDIAADDRESS varchar(64);
declare variable MEDIAPORTS varchar(128);
declare variable CAPACITY integer;
declare variable HOST_SIP_IP varchar(64);
declare variable UACREGISTERCOUNT integer;
declare variable UACSUBSCRIBERCOUNT integer;
declare variable LOGIN varchar(128);
declare variable REG_PASSWORD varchar(128);
declare variable RENEW_INTERVAL integer;
declare variable HOST_REGISTER varchar(128);
declare variable EVENT_NAME varchar(64);
declare variable CONTACT varchar(128);
declare variable LOCALADDRESS varchar(64);
declare variable LOCALPORT integer;
declare variable REMOTEADDRESS varchar(64);
declare variable REMOTEPORT integer;
declare variable LOG_LEVEL integer;
declare variable MAIN_PORT integer;
BEGIN
  if (conf_type not in (-1,0,1)) then
  begin
    result='INVALID CONF_TYPE';
    suspend;
    exit;
  end
  if (not exists
     (select *
        from tb_sys_elements
        where host_id=:host_id and se_type_id=1)) then
  begin
    result='TERMINAL NOT EXIST';
    suspend;
    exit;
  end
  select vc_version
    from tb_ui_conf
    where ui_conf_version=:ui_conf_version
    into :voice_version;
  if (voice_version is null) then
  begin
    result='UI_CONF_VERSION NOT EXIST';
    suspend;
    exit;
  end
  select int_value
    from db_options
    where option_name='SYSTEM_CONF_VERSION'
    into :SYSTEM_CONF_VERSION;

  if (conf_type = 1) then
  begin
    select ui_conf_cfg
      from tb_ui_conf
      where ui_conf_version=:ui_conf_version
      into :terminal_conf_cfg;
  end

  if (conf_type = 0) then
  begin
    select int_value
      from db_options
      where option_name = 'LOG_LEVEL'
      into :log_level;

    select str_value
      from db_options
      where option_name = 'SYSLOG_ADDRESS'
      into :syslog_address;

    select int_value
      from db_options
      where option_name = 'MAIN_PORT'
      into :main_port;

    TERMINAL_CONF_CFG = '[MAIN]
'||'LogLevel='||:log_level||'
'||'LogAddress='||:syslog_address||'
'||'MainInPort='||:main_port||'
'||'ConfVersion='||:system_conf_version||'
'||'UIConfVersion='||:ui_conf_version||'
'||'VoiceVersion='||:voice_version||'

';
    select str_value
      from db_options
      where option_name = 'SNMP_AGENT_ADDRESS'
      into :SNMP_AGENT;

    select str_value
      from db_options
      where option_name = 'SNMP_RECEIVER_ADDRESS'
      into :SNMP_RECEIVER;

    TERMINAL_CONF_CFG = TERMINAL_CONF_CFG||'[SNMP]
'||'AgentAddress='||:SNMP_AGENT||'
'||'ReceiverAddresses='||:SNMP_RECEIVER||'

';
    for
      select channel_id
        from tb_ui_channels
        where ui_conf_version = :ui_conf_version
        order by channel_id
        into :channel_id
    do
    begin
      select logic,re_type_id
        from tb_terminal_channels
        where channel_id = :channel_id
        into :logic,:re_type_id;
      if (re_type_id is not null) then
      begin
        select a.local_desc
          from  tb_terminal_local_devs a
          left join tb_removable_elements b on (a.re_id = b.re_id)
          where b.host_id = :host_id and a.channel_id = :channel_id
          into :local_desc;
        if (local_desc is not null) then
        begin
          if (logic = 'LIRA') then
            TERMINAL_CONF_CFG = :TERMINAL_CONF_CFG||
'['||:logic||'.'||:channel_id||']
'||'Location='||:local_desc||'

';
          if (logic = 'AUDIO') then
            TERMINAL_CONF_CFG = :TERMINAL_CONF_CFG||
'['||:logic||'.'||:channel_id||']
'||'InDevice='||:local_desc||'
'||'OutDevice='||:local_desc||'

';
          TERMINAL_CONF_CFG = :TERMINAL_CONF_CFG||
'[Channel.'||:channel_id||']
'||'Connection='||:logic||'.'||:channel_id||'

';
        end
        else
        begin
          if (result is null) then result = 'ERROR: '||:channel_id;
          else result = :result||', '||:channel_id;
        end
      end
      if (logic = 'SIP') then
      begin
        select port,urischeme,mediaaddress,mediaports,capacity
          from tb_terminal_sip_endp
          where endp_channel_id = :channel_id
          into :port,:urischeme,:mediaaddress,:mediaports,:capacity;
        select host_sip_ip
          from tb_terminal_sip_ip
          where endp_channel_id = :channel_id and host_id = :host_id
          into :host_sip_ip;

        TERMINAL_CONF_CFG = :TERMINAL_CONF_CFG||
'[SIP.'||:channel_id||']
'||'Capacity='||:capacity||'
'||'ReceiverAddress='||:host_sip_ip||':'||:port||'
'||'URIScheme='||:urischeme||'
'||'MediaPorts='||:mediaports||'
'||'MediaAddress='||:mediaaddress||'
';
        uacregistercount = 0;
        for
          select login,reg_password,renew_interval,host_register
            from tb_terminal_sip_regs
            where endp_channel_id = :channel_id
            into :login,:reg_password,:renew_interval,:host_register
        do
        begin
          TERMINAL_CONF_CFG = :TERMINAL_CONF_CFG||
'UACRegister'||:uacregistercount||'='||:login||':'||:reg_password||':'||:renew_interval||'@'||:host_register||'
';
          uacregistercount = uacregistercount + 1;
        end
        TERMINAL_CONF_CFG = :TERMINAL_CONF_CFG||'UACRegisterCount='||:uacregistercount||'
';
        uacsubscribercount = 0;
        for
          select event_name,renew_interval,contact
            from tb_terminal_sip_subs
            where endp_channel_id = :channel_id
            into :event_name,:renew_interval,:contact
        do
        begin
          TERMINAL_CONF_CFG = :TERMINAL_CONF_CFG||
'UACSubscriber'||:uacsubscribercount||'='||:event_name||':'||:renew_interval||':'||:contact||'
';
          uacsubscribercount = uacsubscribercount + 1;
        end
        TERMINAL_CONF_CFG = :TERMINAL_CONF_CFG||'UACSubscriberCount='||:uacsubscribercount||'

';
        TERMINAL_CONF_CFG = :TERMINAL_CONF_CFG||'[Channel.'||:channel_id||']
'||'Connection=SIP.'||:channel_id||'

';

      end
      if (logic = 'LIRA-RADIO') then
      begin
        select localaddress,localport,remoteaddress,remoteport
          from tb_terminal_radio
          where radio_channel_id = :channel_id
          into :localaddress,:localport,:remoteaddress,:remoteport;

        TERMINAL_CONF_CFG = :TERMINAL_CONF_CFG||'[RADIO-LIRA.'||:channel_id||']
'||'LocalAddress='||:localaddress||':'||:localport||'
'||'RemoteAddress='||:remoteaddress||':'||:remoteport||'

'||'[Channel.'||:channel_id||']
'||'Connection=RADIO-LIRA.'||:channel_id||'

';
      end
    end
    select count(channel_id)
      from tb_ui_channels
      where ui_conf_version = :ui_conf_version
      into :capacity;
    if (capacity = 0) then
      capacity = 1;
    if (channel_id is null) then
      channel_id = 0;
    channel_id = channel_id + 1;
    TERMINAL_CONF_CFG = :TERMINAL_CONF_CFG||'[FILE.'||:channel_id||']
'||'Capacity='||:capacity||'

'||'[Channel.'||:channel_id||']
'||'Connection=FILE.'||:channel_id||'

';
  end
  if (result is null) then
    result='SUCCESS';
  SUSPEND;
END
^

CREATE OR ALTER PROCEDURE SAVE_OUTER_GW_CONF(IN_OUTER_GW_ID INTEGER,
UI_CONF_VERSION INTEGER,
CALLER_ID VARCHAR(128),
ACCESS INTEGER)
 RETURNS(NEW_UI_CONF_VERSION INTEGER,
RESULT VARCHAR(64))
 AS
declare variable ui_conf_loading integer;
declare variable host_id integer;
declare variable ui_conf_cfg blob sub_type 1 segment size 80;
declare variable ui_conf_name varchar(128);
declare variable user_id integer;
declare variable add_info varchar(256);
declare variable vc_version integer;
declare variable gw_caller_id varchar(128);
declare variable tmp_outer_gw_id integer;
declare variable tmp_channel_id integer;
begin
  result = 'FAIL';
  if (not exists(select *
                 from tb_ui_conf
                 where ui_conf_version = :ui_conf_version)) then
  begin
    suspend;
    exit;
  end
  if (not exists(select *
                 from tb_outer_gw
                 where outer_gw_id = :in_outer_gw_id)) then
  begin
    suspend;
    exit;
  end

  new_ui_conf_version = ui_conf_version;

  for select host_id
      from tb_sys_elements
      where se_type_id = 1
      into :host_id
  do
  begin
    select first 1 conf_version
    from tb_terminal_loading a
    where conf_type = 1 and
          host_id = :host_id /*and load_time is not null*/
    order by load_id desc
    into :ui_conf_loading;
    if (ui_conf_loading = ui_conf_version) then
    begin
      select ui_conf_name,
             user_id,
             add_info,
             ui_conf_cfg,
             vc_version
      from tb_ui_conf
      where ui_conf_version = :ui_conf_version
      into :ui_conf_name,
           :user_id,
           :add_info,
           :ui_conf_cfg,
           :vc_version;

      new_ui_conf_version = gen_id(gn_ui_version, 1);
      insert into tb_ui_conf
      values (:new_ui_conf_version, :ui_conf_name, 'now', :user_id, :add_info, :ui_conf_cfg, :vc_version);

      for select outer_gw_id,
                 caller_id
          from tb_outer_gw_conf
          where ui_conf_version = :ui_conf_version
          into :tmp_outer_gw_id,
               :gw_caller_id
      do
      begin
        insert into tb_outer_gw_conf (outer_gw_id, ui_conf_version, caller_id)
        values (:tmp_outer_gw_id, :new_ui_conf_version, :gw_caller_id);
      end

      for select channel_id
          from tb_ui_channels
          where ui_conf_version = :ui_conf_version
          into :tmp_channel_id
      do
      begin
        insert into tb_ui_channels
        values (:new_ui_conf_version, :tmp_channel_id);
      end

      select result
      from set_terminal_conf(:host_id, :new_ui_conf_version, 1, null)
      into :result;
    end
  end

  if (access = 1) then
  begin
    update or insert into tb_outer_gw_conf (outer_gw_id, ui_conf_version, caller_id)
    values (:in_outer_gw_id, :new_ui_conf_version, :caller_id);
    result = 'INS or UDT';
  end
  else
  begin
    delete
    from tb_outer_gw_conf
    where outer_gw_id = :in_outer_gw_id and
          ui_conf_version = :new_ui_conf_version;
    result = 'DEL';
  end
  suspend;
end
^

SET TERM ; ^

COMMIT WORK;

